!===============================================================================
! Copyright 2006-2016 Intel Corporation All Rights Reserved.
!
! The source code,  information  and material  ("Material") contained  herein is
! owned by Intel Corporation or its  suppliers or licensors,  and  title to such
! Material remains with Intel  Corporation or its  suppliers or  licensors.  The
! Material  contains  proprietary  information  of  Intel or  its suppliers  and
! licensors.  The Material is protected by  worldwide copyright  laws and treaty
! provisions.  No part  of  the  Material   may  be  used,  copied,  reproduced,
! modified, published,  uploaded, posted, transmitted,  distributed or disclosed
! in any way without Intel's prior express written permission.  No license under
! any patent,  copyright or other  intellectual property rights  in the Material
! is granted to  or  conferred  upon  you,  either   expressly,  by implication,
! inducement,  estoppel  or  otherwise.  Any  license   under such  intellectual
! property rights must be express and approved by Intel in writing.
!
! Unless otherwise agreed by Intel in writing,  you may not remove or alter this
! notice or  any  other  notice   embedded  in  Materials  by  Intel  or Intel's
! suppliers or licensors in any way.
!===============================================================================

! Content:
!       MKL DFTI implementation through FFTW interface (via wrappers) example
!       program (Fortran-interface)
!
! Real-to-complex and complex-to-real 3D transform for REAL*4 and COMPLEX*8
! data not inplace with strides and threads.
!
! Configuration parameters for MKL DFTI:
!           DFTI_FORWARD_DOMAIN = DFTI_REAL        (obligatory)
!           DFTI_PRECISION      = DFTI_SINGLE      (obligatory)
!           DFTI_DIMENSION      = 3                (obligatory)
!           DFTI_LENGTHS        = {n,m,l}          (obligatory)
!           DFTI_PLACEMENT      = DFTI_NOT_INPLACE (default=DFTI_INPLACE)
!           DFTI_FORWARD_SCALE  = 1.0              (default)
!           DFTI_BACKWARD_SCALE = 1.0/(n*m*l)      (default=1.0)
!
! Other default configuration parameters are in the mkl_dfti.f90 interface file
!*****************************************************************************

      PROGRAM REAL_3D_SINGLE_EX8

      INCLUDE 'fftw_f77.i'
      INCLUDE 'mkl_fftw_examples.fi'

      INTEGER SIZE,N,M,L,OUT_N,NTHREADS,STATUS,IDIST,ODIST,ISTRIDE
      INTEGER OSTRIDE,HOWMANY,I
      PARAMETER (L=3)
      PARAMETER (M=5)
      PARAMETER (N=8)
      PARAMETER (OUT_N=INT(N/2.0)+1)
      PARAMETER (ISTRIDE=2)
      PARAMETER (OSTRIDE=3)
      PARAMETER (IDIST=N*M*L*ISTRIDE+3)
      PARAMETER (ODIST=OUT_N*M*L*OSTRIDE+5)
      PARAMETER (HOWMANY=2)
      PARAMETER (SIZE=HOWMANY*(IDIST+ODIST))
      PARAMETER (NTHREADS=2)
      INTEGER*8 MY_PLAN

      REAL*4 IN(SIZE),EXP_X(SIZE)
      COMPLEX*8 OUT(SIZE)
      REAL*4 ERR,SCALE

!
!     Initialize IN and copy to expected EXP_X
!
      PRINT *, 'Initialize data array'
      DO I=1,SIZE
        IN(I)=SIN(FLOAT(I))
        EXP_X(I)=IN(I)
      END DO

!
!     FFTW requires user to call this routine before calling any other FFTW routines
!
      CALL FFTW_F77_THREADS_INIT(STATUS)

!
!     Create FFTW plan for 3D real to complex transform
!
      PRINT *, 'Create FFTW plan for 3D real to complex transform'
      CALL RFFTW3D_F77_CREATE_PLAN(MY_PLAN,N,M,L,FFTW_REAL_TO_COMPLEX,
     *                 FFTW_ESTIMATE)

!
!     Compute 3D real to complex transform
!
      PRINT *, 'Compute 3D real to complex transform'
      CALL RFFTWND_F77_THREADS_REAL_TO_COMPLEX(NTHREADS,MY_PLAN,
     *                 HOWMANY,IN,ISTRIDE,IDIST,OUT,OSTRIDE,ODIST)

!
!     Destroy FFTW plan
!
      PRINT *, 'Destroy FFTW plan'
      CALL RFFTWND_F77_DESTROY_PLAN(MY_PLAN)

!
!     Create FFTW plan for 3D complex to real transform
!
      PRINT *, 'Create FFTW plan for 3D complex to real transform'
      CALL RFFTW3D_F77_CREATE_PLAN(MY_PLAN,N,M,L,FFTW_COMPLEX_TO_REAL,
     *                 FFTW_ESTIMATE)

!
!     Compute 3D complex to real transform
!
      PRINT *, 'Compute 3D complex to real transform'
      CALL RFFTWND_F77_THREADS_COMPLEX_TO_REAL(NTHREADS,MY_PLAN,
     *                 HOWMANY,OUT,OSTRIDE,ODIST,IN,ISTRIDE,IDIST)

!
!     Destroy FFTW plan
!
      PRINT *, 'Destroy FFTW plan'
      CALL RFFTWND_F77_DESTROY_PLAN(MY_PLAN)

!
!     Scale result. FFTW can't do this itself.
!
      PRINT *, 'Scale result by 1/(N*M*L)'
      SCALE=1.0/REAL((N*M*L),4)
      CALL SCALE_WITH_STRIDES_S(IN,SCALE,HOWMANY,N*M*L,ISTRIDE,IDIST)

!
!     Check results
!
      PRINT *, 'Check results'
      CALL CHECK_RESULT_S(IN,EXP_X,SIZE,ERR)
      PRINT *, 'Accuracy=',ERR
      IF (ERR .GT. MAX_SINGLE_ERR) THEN
       PRINT *, 'TEST FAILED'
       STOP 1
      END IF
      PRINT *, 'TEST PASSED'

      PRINT *, 'END OF TEST'

      END PROGRAM
