!===============================================================================
! Copyright 2005-2016 Intel Corporation All Rights Reserved.
!
! The source code,  information  and material  ("Material") contained  herein is
! owned by Intel Corporation or its  suppliers or licensors,  and  title to such
! Material remains with Intel  Corporation or its  suppliers or  licensors.  The
! Material  contains  proprietary  information  of  Intel or  its suppliers  and
! licensors.  The Material is protected by  worldwide copyright  laws and treaty
! provisions.  No part  of  the  Material   may  be  used,  copied,  reproduced,
! modified, published,  uploaded, posted, transmitted,  distributed or disclosed
! in any way without Intel's prior express written permission.  No license under
! any patent,  copyright or other  intellectual property rights  in the Material
! is granted to  or  conferred  upon  you,  either   expressly,  by implication,
! inducement,  estoppel  or  otherwise.  Any  license   under such  intellectual
! property rights must be express and approved by Intel in writing.
!
! Unless otherwise agreed by Intel in writing,  you may not remove or alter this
! notice or  any  other  notice   embedded  in  Materials  by  Intel  or Intel's
! suppliers or licensors in any way.
!===============================================================================

!  Content: Intel MKL RCI (P)CG Fortran example
!
!*******************************************************************************

!---------------------------------------------------------------------------
!  Example program for solving symmetric positive definite system of equations.
!  Simplest case: no preconditioning and no the user-defined stopping tests.
!---------------------------------------------------------------------------
      PROGRAM rci_pcgmrhs_f_test_1
        IMPLICIT NONE

        INCLUDE 'mkl_rci.fi'
!---------------------------------------------------------------------------
! Define arrays for the upper triangle of the coefficient matrix and rhs vector
! Compressed sparse row storage is used for sparse representation
!---------------------------------------------------------------------------
        INTEGER N, nRhs
        PARAMETER (N=8,nRhs=2)
        INTEGER RCI_request, itercount(nRhs), i
        DOUBLE PRECISION  rhs(N,nRhs)

! Matrix
        INTEGER IA(9)
        INTEGER JA(18)
        DOUBLE PRECISION A(18)
! Fill all arrays containing matrix data.
        DATA IA /1,5,8,10,12,15,17,18,19/
        DATA JA
     1 /1,  3,    6,7,
     2    2,3,  5,
     3      3,        8,
     4        4,    7,
     5          5,6,7,
     6            6,  8,
     7              7,
     8                8/
        DATA A
     1 /7.D0,       1.D0,             2.D0, 7.D0,
     2       -4.D0, 8.D0,       2.D0,
     3              1.D0,                         5.D0,
     4                    7.D0,             9.D0,
     5                          5.D0, 1.D0, 5.D0,
     6                               -1.D0,       5.D0,
     7                                     11.D0,
     8                                            5.D0/
!---------------------------------------------------------------------------
! Allocate storage for the solver ?par and the initial solution vector
!---------------------------------------------------------------------------
        INTEGER length, method
        PARAMETER (length=128, method=1)
        DOUBLE PRECISION expected_sol(N,nRhs)
        DOUBLE PRECISION solution(N,nRhs)

        INTEGER ipar(length + 2*nRhs)
        DOUBLE PRECISION dpar(length + 2*nRhs),TMP(N,3 + nRhs)
!---------------------------------------------------------------------------
! Some additional variables to use with the RCI (P)CG solver
!---------------------------------------------------------------------------
        DATA expected_sol/1.D0, 0.D0, 1.D0, 0.D0, 1.D0, 0.D0, 1.D0, 
     &  0.D0, 0.D0, 2.D0, 0.D0, 2.D0, 0.D0, 2.D0, 0.D0, 2.D0/
        DOUBLE PRECISION DNRM2, Euclidean_norm
        EXTERNAL DNRM2
!---------------------------------------------------------------------------
! Initialize the right hand side through matrix-vector product
!---------------------------------------------------------------------------
        CALL MKL_DCSRSYMV('U', N, A, IA, JA, expected_sol, rhs)
        CALL MKL_DCSRSYMV('U', N, A, IA, JA, expected_sol(1,2), 
     &  rhs(1,2))
!---------------------------------------------------------------------------
! Initialize the initial guess
!---------------------------------------------------------------------------
         DO I = 1, N
             solution(I,1) = 1.D0
             solution(I,2) = 2.D0
         END DO
!---------------------------------------------------------------------------
! Initialize the solver
!---------------------------------------------------------------------------
        DO I = 1, length + 2*nRhs
            ipar(i) = 0;
            dpar(i) = 0;
        END DO

        CALL dcgmrhs_init(N,solution,nRhs,rhs,method,
     &  RCI_request,ipar,dpar,TMP)
        IF (RCI_request .NE. 0 ) GO TO 999
!---------------------------------------------------------------------------
! Set the desired parameters:
! LOGICAL parameters:
! do residual stopping test
! do not request for the user defined stopping test
! DOUBLE PRECISION parameters
! set the relative tolerance to 1.0D-5 instead of default value 1.0D-6
!---------------------------------------------------------------------------
        ipar(9) = 1
        ipar(10) = 0
        dpar(1) = 1.D-5
!---------------------------------------------------------------------------
! Compute the solution by RCI (P)CG solver without preconditioning
! Reverse Communications starts here
!---------------------------------------------------------------------------
1       CALL dcgmrhs(N,solution,nRhs,rhs,RCI_request,ipar,dpar,TMP)
!---------------------------------------------------------------------------
! If RCI_request=0, then the solution was found with the required precision
!---------------------------------------------------------------------------
        IF (RCI_request .EQ. 0) THEN
            GO TO 700
!---------------------------------------------------------------------------
! If RCI_request=1, then compute the vector A*TMP(:,1)
! and put the result in vector TMP(:,2)
!---------------------------------------------------------------------------
        ELSE IF (RCI_request .EQ. 1) THEN
            CALL MKL_DCSRSYMV('U', N, A, IA, JA, TMP, TMP(1,2))
            GO TO 1
        ELSE
!---------------------------------------------------------------------------
! If RCI_request=anything else, then dcg subroutine failed
! to compute the solution vector: solution(N)
!---------------------------------------------------------------------------
            GO TO 999
        END IF
!---------------------------------------------------------------------------
! Reverse Communication ends here
! Get the current iteration number
!---------------------------------------------------------------------------
700     CALL dcgmrhs_get(N,solution,nRhs,rhs,RCI_request,ipar,dpar,TMP,
     &  itercount)
!---------------------------------------------------------------------------
! Print solution vector: solution(N) and number of iterations: itercount
!---------------------------------------------------------------------------
        WRITE(*, *) ' The system has been solved '
        WRITE(*, *) ' The following solution obtained '
        WRITE(*,800) (solution(i,1), i=1,N)
        WRITE(*,800) (solution(i,2), i=1,N)
        WRITE(*, *) ' expected solution '
        WRITE(*,800)(expected_sol(i,1), i=1,N)
        WRITE(*,800)(expected_sol(i,2), i=1,N)
800     FORMAT(4(F10.3))
        DO i = 1, N
            expected_sol(i,1) = expected_sol(i,1) - solution(i,1)
            expected_sol(i,2) = expected_sol(i,2) - solution(i,2)
        END DO

        Euclidean_norm = DNRM2(2*N,expected_sol,1)

!---------------------------------------------------------------------------
! Release internal MKL memory that might be used for computations
! NOTE: It is important to call the routine below to avoid memory leaks
! unless you disable MKL Memory Manager
!---------------------------------------------------------------------------
        CALL MKL_FREE_BUFFERS

        IF (Euclidean_norm .LE. 1.0D-12) THEN
            WRITE( *,'(A,A)') 'This example has successfully PASSED',
     &  ' through all steps of computation!'
            STOP 0
        ELSE
            WRITE( *,'(A,A,A,E12.5,A)') 'This example may have',
     &  ' FAILED as the computed solution differs much from the',
     &  ' expected solution (Euclidean norm is',Euclidean_norm,').'
            STOP 1
        END IF
!---------------------------------------------------------------------------
! Release internal MKL memory that might be used for computations
! NOTE: It is important to call the routine below to avoid memory leaks
! unless you disable MKL Memory Manager
!---------------------------------------------------------------------------
999     WRITE( *,'(A,A)') 'This example FAILED as the solver has',
     &  ' returned the ERROR code', RCI_request
        CALL MKL_FREE_BUFFERS
        STOP 1

      END PROGRAM rci_pcgmrhs_f_test_1
