!===============================================================================
! Copyright 2004-2016 Intel Corporation All Rights Reserved.
!
! The source code,  information  and material  ("Material") contained  herein is
! owned by Intel Corporation or its  suppliers or licensors,  and  title to such
! Material remains with Intel  Corporation or its  suppliers or  licensors.  The
! Material  contains  proprietary  information  of  Intel or  its suppliers  and
! licensors.  The Material is protected by  worldwide copyright  laws and treaty
! provisions.  No part  of  the  Material   may  be  used,  copied,  reproduced,
! modified, published,  uploaded, posted, transmitted,  distributed or disclosed
! in any way without Intel's prior express written permission.  No license under
! any patent,  copyright or other  intellectual property rights  in the Material
! is granted to  or  conferred  upon  you,  either   expressly,  by implication,
! inducement,  estoppel  or  otherwise.  Any  license   under such  intellectual
! property rights must be express and approved by Intel in writing.
!
! Unless otherwise agreed by Intel in writing,  you may not remove or alter this
! notice or  any  other  notice   embedded  in  Materials  by  Intel  or Intel's
! suppliers or licensors in any way.
!===============================================================================

!   Content : MKL PARDISO Fortran-90 example
!
!*******************************************************************************
!----------------------------------------------------------------------
! Example program to show the use of the "matrix_check" routine
! for symmetric linear systems
!---------------------------------------------------------------------

INCLUDE 'mkl_sparse_handle.f90'

PROGRAM MATRIX_CHECK

    USE MKL_SPARSE_HANDLE
    IMPLICIT NONE

    INTEGER N, NNZ, CHECK_RESULT_CODE, ERROR
    INTEGER, ALLOCATABLE :: IA(:), JA(:)
    TYPE(SPARSE_STRUCT) PT

    N   = 8
    NNZ = 18
    ERROR = 0
    ALLOCATE(IA(N + 1))
    IA = (/ 1, 5, 8, 10, 12, 15, 17, 18, 19 /)
    ALLOCATE(JA(NNZ))
    JA = (/ 1, 3,       6, 7,        &
            2, 3,    5,              &
               3,              8,    &
               3,          7,        &
                     5, 6, 7,        &
                        6,     8,    &
                            7,       &
                               8 /)

    CALL SPARSE_MATRIX_CHECKER_INIT(PT)

    PT % N = N
    PT % CSR_IA = LOC(IA)
    PT % CSR_JA = LOC(JA)
    PT % INDEXING         = MKL_ONE_BASED
    PT % MATRIX_STRUCTURE = MKL_UPPER_TRIANGULAR
    PT % PRINT_STYLE      = MKL_C_STYLE
    PT % MESSAGE_LEVEL    = MKL_PRINT

    CHECK_RESULT_CODE = SPARSE_MATRIX_CHECKER(PT)


    IF (CHECK_RESULT_CODE .EQ. MKL_SPARSE_CHECKER_NONTRIANGULAR) THEN
        WRITE(*,*)'Matrix check result: MKL_SPARSE_CHECKER_NONTRIANGULAR'
        ERROR = 0
    ELSE
        IF (CHECK_RESULT_CODE .EQ. MKL_SPARSE_CHECKER_NON_MONOTONIC) THEN
            WRITE(*,*)'Matrix check result: MKL_SPARSE_CHECKER_NON_MONOTONIC'
        ENDIF
        IF (CHECK_RESULT_CODE .EQ. MKL_SPARSE_CHECKER_SUCCESS) THEN
            WRITE(*,*)'Matrix check result: MKL_SPARSE_CHECKER_SUCCESS'
        ENDIF
        IF (CHECK_RESULT_CODE .EQ. MKL_SPARSE_CHECKER_OUT_OF_RANGE) THEN
            WRITE(*,*)'Matrix check result: MKL_SPARSE_CHECKER_OUT_OF_RANGE'
        ENDIF
        IF (CHECK_RESULT_CODE .EQ. MKL_SPARSE_CHECKER_NONORDERED) THEN
            WRITE(*,*)'Matrix check result: MKL_SPARSE_CHECKER_NONORDERED'
        ENDIF
        ERROR = 1
        ENDIF

    WRITE(*,'("Matrix check details: (",i0,", ",i0,", ",i0,")")') PT % CHECK_RESULT

        IF (ERROR .NE. 0) STOP 1

END PROGRAM MATRIX_CHECK
