!===============================================================================
! Copyright 2001-2016 Intel Corporation All Rights Reserved.
!
! The source code,  information  and material  ("Material") contained  herein is
! owned by Intel Corporation or its  suppliers or licensors,  and  title to such
! Material remains with Intel  Corporation or its  suppliers or  licensors.  The
! Material  contains  proprietary  information  of  Intel or  its suppliers  and
! licensors.  The Material is protected by  worldwide copyright  laws and treaty
! provisions.  No part  of  the  Material   may  be  used,  copied,  reproduced,
! modified, published,  uploaded, posted, transmitted,  distributed or disclosed
! in any way without Intel's prior express written permission.  No license under
! any patent,  copyright or other  intellectual property rights  in the Material
! is granted to  or  conferred  upon  you,  either   expressly,  by implication,
! inducement,  estoppel  or  otherwise.  Any  license   under such  intellectual
! property rights must be express and approved by Intel in writing.
!
! Unless otherwise agreed by Intel in writing,  you may not remove or alter this
! notice or  any  other  notice   embedded  in  Materials  by  Intel  or Intel's
! suppliers or licensors in any way.
!===============================================================================

! Content:
! Example of 2-dimension linear correlation operation on double precision data
!*******************************************************************************/

      INCLUDE 'mkl_vsl.fi'

      PROGRAM vsldcorrelation_2d_fft
      USE MKL_VSL_TYPE
      USE MKL_VSL
      IMPLICIT NONE
C
      INTEGER status,i,j
C
      TYPE(VSL_CORR_TASK) task
      INTEGER mode,rank,xshape(2),yshape(2),zshape(2)
      INTEGER xstride(2),ystride(2),zstride(2)
      REAL*8 x(3,2),y(4,3),z(6,4),e(6,4)
      DATA x/1,1,1, 1,1,1/
      DATA y/1,1,1,1, 1,1,1,1, 1,1,1,1/
      DATA z/0,0,0,0,0,0, 0,0,0,0,0,0, 0,0,0,0,0,0, 0,0,0,0,0,0/
      DATA e/1,2,3,3,2,1, 2,4,6,6,4,2, 2,4,6,6,4,2, 1,2,3,3,2,1/
      DATA xstride/1,3/, ystride/1,4/, zstride/1,6/
      DATA xshape/3,2/,  yshape/4,3/,  zshape/6,4/
      DATA rank/2/

      mode = VSL_CORR_MODE_FFT

C
C  Create task descriptor (create descriptor of problem)
C
      status = vsldcorrnewtask(task,mode,rank,xshape,yshape,zshape)
      IF (status .NE. VSL_STATUS_OK) THEN
          PRINT *,"ERROR: creation of job failed, exit with ", status
          STOP 1
      ENDIF
C
C  Execute task (Calculate 2 dimension correlation of two arrays)
C
      status = vsldcorrexec(task,x,xstride,y,ystride,z,zstride)
      IF (status .NE. VSL_STATUS_OK) THEN
          PRINT *,"ERROR: job status bad, exit with", status
          STOP 1
      ENDIF
C
C  Delete task object (delete descriptor of problem)
C
      status = vslcorrdeletetask(task)
      IF (status .NE. VSL_STATUS_OK) THEN
          PRINT *,"ERROR: failed to delete task, exit with", status
          STOP 1
      ENDIF
C
C  Check results for correctness:
C
      DO j=1,zshape(2)
        DO i=1,zshape(1)
            IF (dabs(z(i,j)-e(i,j)) .GT. e(i,j)*1d-10) THEN
                PRINT *, "ERROR:"
                PRINT *, "        i :", i
                PRINT *, "        j :", j
                PRINT *, "    z(i,j):", z(i,j)
                PRINT *, "  expected:", e(i,j)
                PRINT *, "EXAMPLE FAILED"
                STOP 1
             END IF
        END DO
      END DO
C
      PRINT *, "EXAMPLE PASSED"
      STOP 0
      END
