!===============================================================================
! Copyright 2003-2016 Intel Corporation All Rights Reserved.
!
! The source code,  information  and material  ("Material") contained  herein is
! owned by Intel Corporation or its  suppliers or licensors,  and  title to such
! Material remains with Intel  Corporation or its  suppliers or  licensors.  The
! Material  contains  proprietary  information  of  Intel or  its suppliers  and
! licensors.  The Material is protected by  worldwide copyright  laws and treaty
! provisions.  No part  of  the  Material   may  be  used,  copied,  reproduced,
! modified, published,  uploaded, posted, transmitted,  distributed or disclosed
! in any way without Intel's prior express written permission.  No license under
! any patent,  copyright or other  intellectual property rights  in the Material
! is granted to  or  conferred  upon  you,  either   expressly,  by implication,
! inducement,  estoppel  or  otherwise.  Any  license   under such  intellectual
! property rights must be express and approved by Intel in writing.
!
! Unless otherwise agreed by Intel in writing,  you may not remove or alter this
! notice or  any  other  notice   embedded  in  Materials  by  Intel  or Intel's
! suppliers or licensors in any way.
!===============================================================================

!  Content:
!    Sorting data array  Example Program Text
!******************************************************************************/

      include 'mkl_vsl.f90'
      include "errcheck.inc"
      include "generatedata.inc"
      include "statchars.inc"

      program MKL_VSL_TEST

      USE MKL_VSL_TYPE
      USE MKL_VSL

      integer,parameter :: DIM = 3      ! Task dimension
      integer,parameter :: NN  = 10     ! Number of observations

      TYPE(VSL_STREAM_STATE) :: stream
      integer,parameter :: SEED   = 1
      integer,parameter :: BRNG   = VSL_BRNG_MCG31
      integer,parameter :: METHOD = VSL_RNG_METHOD_UNIFORM_STD

      TYPE(VSL_SS_TASK) task
      integer p
      integer n
      integer x_storage
      integer y_storage
      real(kind=8) x(NN, DIM)
      real(kind=8) y(NN, DIM)
      real(kind=8) lBound, rBound
      integer i, j, errcode
      integer errnums
      integer task_method
      integer edit_value
      integer(kind=8) task_params

!     ***** Initialize parameters *****
      p           = DIM
      n           = NN
      x_storage   = VSL_SS_MATRIX_STORAGE_ROWS
      y_storage   = VSL_SS_MATRIX_STORAGE_ROWS
      lBound      = 0.0
      rBound      = 10.0
      task_params = VSL_SS_SORTED_OBSERV
      task_method = VSL_SS_METHOD_RADIX
      edit_value  = VSL_SS_ED_SORTED_OBSERV_STORAGE
      errcode     = 0
      errnums     = 0

!     ***** Generate data set using Uniforn RNG *****
!     ***** Initialize *****
      errcode = vslnewstream( stream, BRNG,  SEED )
      call CheckVslError(errcode)

!     ***** Call RNG *****
      errcode = vdrnguniform(METHOD, stream, n * p, x, lBound, rBound)
      call CheckVslError(errcode)

!     ***** Create Summary Statistics task *****
      errcode = vsldssnewtask( task, p, n, x_storage, x )
      call CheckVslError( errcode )

      errcode = vsldssedittask( task, VSL_SS_ED_SORTED_OBSERV, y )
      call CheckVslError( errcode )

      errcode = vslissedittask( task, edit_value, y_storage )
      call CheckVslError( errcode )

!     ***** Sort data using RADIX method  *****
      errcode = vsldsscompute( task, task_params, task_method )
      call CheckVslError( errcode )

!     ***** Check the correctness of sorting *****
      do j = 1, p
        do i = 1, n-1
          if( y(i, j) >= y(i + 1, j) ) then
            errnums = errnums + 1
          end if
        end do
      end do

!     ***** Printing results *****
      print *, ''
      print *, 'Task dimension :        ', p
      print *, 'Number of observations :', n
      print *, ''

!     ***** Printing of the initial matrix *****
      print *,'Initial matrix'
      do j = 1, p
        do i = 1, n
           write(*, 5) x(i,j), ' '
        end do
        print *, ''
      end do


!     ***** Printing of the sorted matrix *****
      print *,'Sorted matrix:'
      do j = 1, p
        do i = 1, n
          write(*, 5) y(i,j), ' '
        end do
        print *, ''
      end do

      print *, ''

!     ***** Printing summary of the test *****
      if( errnums == 0 ) then
        print *, 'Sorting is correct'
      else
        print *, 'Error: Sorting is incorrect'
        stop 1
      end if

!     ***** Delete Summary Statistics task *****
      errcode = vslssdeletetask( task )
      call CheckVslError( errcode )

      call MKL_FREE_BUFFERS()

5     format (F6.3,A,$)

      end
