!===============================================================================
! Copyright 2003-2016 Intel Corporation All Rights Reserved.
!
! The source code,  information  and material  ("Material") contained  herein is
! owned by Intel Corporation or its  suppliers or licensors,  and  title to such
! Material remains with Intel  Corporation or its  suppliers or  licensors.  The
! Material  contains  proprietary  information  of  Intel or  its suppliers  and
! licensors.  The Material is protected by  worldwide copyright  laws and treaty
! provisions.  No part  of  the  Material   may  be  used,  copied,  reproduced,
! modified, published,  uploaded, posted, transmitted,  distributed or disclosed
! in any way without Intel's prior express written permission.  No license under
! any patent,  copyright or other  intellectual property rights  in the Material
! is granted to  or  conferred  upon  you,  either   expressly,  by implication,
! inducement,  estoppel  or  otherwise.  Any  license   under such  intellectual
! property rights must be express and approved by Intel in writing.
!
! Unless otherwise agreed by Intel in writing,  you may not remove or alter this
! notice or  any  other  notice   embedded  in  Materials  by  Intel  or Intel's
! suppliers or licensors in any way.
!===============================================================================

!  Content:
!    Calculation of group/poopled means  Example Program Text
!******************************************************************************/

      include 'mkl_vsl.f90'
      include "errcheck.inc"
      include "generatedata.inc"
      include "statchars.inc"

      program MKL_VSL_TEST

      USE MKL_VSL_TYPE
      USE MKL_VSL

      integer,parameter :: DIM = 3       ! Task dimension
      integer,parameter :: NN  = 10000   ! Number of observations
      integer,parameter :: GG  = 2       ! Number of groups
      integer,parameter :: GGN = 2       ! Number of group means

      real(kind=4),parameter :: P_THRESHOLD = 0.005

      real(kind=4) C(DIM*DIM)            ! Exact covariance matrix
      data C  / 1.0, 0.0, 0.0,                                          &
     &          0.0, 1.0, 0.0,                                          &
     &          0.0, 0.0, 1.0 /

      real(kind=4) m(DIM)                ! Exact means vector
      data m  / 0.0, 0.0, 0.0 /

      integer group_mean_indices(GG)
      data group_mean_indices / 1, 1 /

      TYPE(VSL_SS_TASK) task
      integer p
      integer n
      integer x_storage
      integer pld_cov_storage
      integer grp_cov_storage
      real(kind=4) x(NN,DIM)
      real(kind=4) pld_mean(DIM)
      real(kind=4),target :: grp_mean(DIM*GGN)
      real(kind=4),pointer :: grp_mean_ptr(:)
      real(kind=4) a, sigma
      integer group_indices(NN)
      integer i, j, errcode
      integer errnums
      integer x_len
      integer task_method
      integer(kind=8) task_params

      real(kind=4) pval_pld_mean(DIM)
      real(kind=4),target :: pval_grp_mean(DIM*GGN)
      real(kind=4),pointer :: pval_grp_mean_ptr(:)

!     ***** Generate data set using VSL Gaussian RNG
!           with a = 0 and sigma = 1 *****

      p               = DIM
      n               = NN
      a               = 0.0
      sigma           = 1.0
      errcode = sGenerateGaussianData(  p, n, x, a, sigma )
      call CheckVslError( errcode )

!     Dividing elements into odd and even
      do i = 1, n
        group_indices(i) = mod(i - 1, 2)
      end do

!     ***** Initialize parameters of Summary Statistics task *****
      p               = DIM
      n               = NN
      x_storage       = VSL_SS_MATRIX_STORAGE_ROWS
      task_params     = ior( VSL_SS_GROUP_MEAN, VSL_SS_POOLED_MEAN )
      task_method     = VSL_SS_METHOD_1PASS
      errcode         = 0

!     ***** Create Summary Statistics task *****
      errcode  = vslsssnewtask( task, p, n, x_storage, x )
      call CheckVslError( errcode )

      errcode = vslssseditpooledcovariance( task, group_indices,        &
     &      pld_mean, grp_cov_indices=group_mean_indices,               &
     &      grp_means=grp_mean )
      call CheckVslError( errcode )

!     ***** Compute means using 1PASS method  *****
      errcode = vslssscompute( task, task_params, task_method )
      call CheckVslError( errcode )

!     ***** Testing stat characteristics of means *****
!     Compute p-values for group mean estimates
      call sComputePvalsMean( p, n, grp_mean, m, C, pval_grp_mean )
      grp_mean_ptr      => grp_mean( p+1 : 2*p )
      pval_grp_mean_ptr => pval_grp_mean( p+1 : 2*p )
      call sComputePvalsMean( p, n, grp_mean_ptr, m, C,                 &
     &                        pval_grp_mean_ptr )

!     Compute p-values for pooled mean estimates
      call sComputePvalsMean( p, n, pld_mean, m, C, pval_pld_mean )

      errnums = 0
      do i = 1, p
        if( pval_grp_mean(i) < P_THRESHOLD ) then
          errnums = errnums + 1
        end if
        if( pval_grp_mean(i + p) < P_THRESHOLD ) then
          errnums = errnums + 1
        end if
        if( pval_pld_mean(i) < P_THRESHOLD ) then
          errnums = errnums + 1
        end if
      end do

!     ***** Printing results *****
      print *, 'Task dimension :         ', p
      print *, 'Number of observations : ', n
      print *, ''

!     ***** Print exact mean *****
      print *, 'Exact means:'
      do i = 1, p
        print 5, m(i), ' '
      end do
      print *, ''
      print *, ''

!     ***** Print group mean estimates *****
      print *, 'Group means:'
      do i = 1, p
        print 5, grp_mean(i), ' '
      end do

      print 6, '     '

      do i = 1, p
        print 5, grp_mean_ptr(i), ' '
      end do
      print *, ''
      print *, ''

!     ***** Print pooled mean estimates *****
      print *, 'Pooled means:'
      do i = 1, p
        print 5, pld_mean(i), ' '
      end do
      print *, ''
      print *, ''
      print *, ''

      print *, 'P-values of the computed group mean:'
      do i = 1, p
        print 5, pval_grp_mean(i), ' '
      end do

      print 6, '     '

      do i = 1, p
        print 5, pval_grp_mean_ptr(i), ' '
      end do
      print *, ''
      print *, ''

      print *, 'P-values of the computed pooled mean:'
      do i = 1, p
        print 5, pval_pld_mean(i), ' '
      end do
      print *, ''
      print *, ''
      print *, ''

!     ***** Delete Summary Statistics task *****
      errcode = vslssdeletetask( task )
      call CheckVslError( errcode )

      call MKL_FREE_BUFFERS()

!     ***** Printing summary of the test *****
      if ( errnums == 0 ) then
        print *, ' Pooled and group mean estimates',                    &
     &           ' agree with theory'
      else
        print *, ' Pooled and group mean estimates',                    &
     &           ' disagree with theory'
        stop 1
      end if

5     format (F9.6,A,$)
6     format (A,$)

      end
